# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from .types import (
    EmailFlag,
    WebhookEventType,
    EmailTry,
    Email,
    DomainRecordsDMARC,
    DomainRecords,
    DomainReputation,
    DomainStatistics,
    Domain,
    Webhook,
    CreateEmailResponse,
    DomainLastStatusDkimRecord,
    DomainLastStatusDmarcRecord,
    DomainLastStatusSpfRecord,
    DomainLastStatus,
    ListDomainsResponse,
    ListEmailsResponse,
    WebhookEvent,
    ListWebhookEventsResponse,
    ListWebhooksResponse,
    Statistics,
    CreateDomainRequest,
    CreateEmailRequestAddress,
    CreateEmailRequestAttachment,
    CreateEmailRequestHeader,
    CreateEmailRequest,
    CreateWebhookRequest,
    UpdateWebhookRequest,
)


def unmarshal_EmailTry(data: Any) -> EmailTry:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'EmailTry' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("rank", None)
    if field is not None:
        args["rank"] = field

    field = data.get("code", None)
    if field is not None:
        args["code"] = field

    field = data.get("message", None)
    if field is not None:
        args["message"] = field

    field = data.get("tried_at", None)
    if field is not None:
        args["tried_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["tried_at"] = None

    return EmailTry(**args)


def unmarshal_Email(data: Any) -> Email:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Email' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("message_id", None)
    if field is not None:
        args["message_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("mail_from", None)
    if field is not None:
        args["mail_from"] = field

    field = data.get("mail_rcpt", None)
    if field is not None:
        args["mail_rcpt"] = field

    field = data.get("rcpt_to", None)
    if field is not None:
        args["rcpt_to"] = field
    else:
        args["rcpt_to"] = None

    field = data.get("rcpt_type", None)
    if field is not None:
        args["rcpt_type"] = field

    field = data.get("subject", None)
    if field is not None:
        args["subject"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("try_count", None)
    if field is not None:
        args["try_count"] = field

    field = data.get("last_tries", None)
    if field is not None:
        args["last_tries"] = (
            [unmarshal_EmailTry(v) for v in field] if field is not None else None
        )

    field = data.get("flags", None)
    if field is not None:
        args["flags"] = [EmailFlag(v) for v in field] if field is not None else None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("status_details", None)
    if field is not None:
        args["status_details"] = field
    else:
        args["status_details"] = None

    return Email(**args)


def unmarshal_DomainRecordsDMARC(data: Any) -> DomainRecordsDMARC:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainRecordsDMARC' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("value", None)
    if field is not None:
        args["value"] = field

    return DomainRecordsDMARC(**args)


def unmarshal_DomainRecords(data: Any) -> DomainRecords:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainRecords' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dmarc", None)
    if field is not None:
        args["dmarc"] = unmarshal_DomainRecordsDMARC(field)
    else:
        args["dmarc"] = None

    return DomainRecords(**args)


def unmarshal_DomainReputation(data: Any) -> DomainReputation:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainReputation' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("score", None)
    if field is not None:
        args["score"] = field

    field = data.get("scored_at", None)
    if field is not None:
        args["scored_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["scored_at"] = None

    field = data.get("previous_score", None)
    if field is not None:
        args["previous_score"] = field
    else:
        args["previous_score"] = None

    field = data.get("previous_scored_at", None)
    if field is not None:
        args["previous_scored_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["previous_scored_at"] = None

    return DomainReputation(**args)


def unmarshal_DomainStatistics(data: Any) -> DomainStatistics:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainStatistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("sent_count", None)
    if field is not None:
        args["sent_count"] = field

    field = data.get("failed_count", None)
    if field is not None:
        args["failed_count"] = field

    field = data.get("canceled_count", None)
    if field is not None:
        args["canceled_count"] = field

    return DomainStatistics(**args)


def unmarshal_Domain(data: Any) -> Domain:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Domain' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("spf_config", None)
    if field is not None:
        args["spf_config"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("next_check_at", None)
    if field is not None:
        args["next_check_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["next_check_at"] = None

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("dkim_config", None)
    if field is not None:
        args["dkim_config"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("revoked_at", None)
    if field is not None:
        args["revoked_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["revoked_at"] = None

    field = data.get("last_error", None)
    if field is not None:
        args["last_error"] = field
    else:
        args["last_error"] = None

    field = data.get("statistics", None)
    if field is not None:
        args["statistics"] = unmarshal_DomainStatistics(field)
    else:
        args["statistics"] = None

    field = data.get("reputation", None)
    if field is not None:
        args["reputation"] = unmarshal_DomainReputation(field)
    else:
        args["reputation"] = None

    field = data.get("records", None)
    if field is not None:
        args["records"] = unmarshal_DomainRecords(field)
    else:
        args["records"] = None

    return Domain(**args)


def unmarshal_Webhook(data: Any) -> Webhook:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Webhook' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("event_types", None)
    if field is not None:
        args["event_types"] = (
            [WebhookEventType(v) for v in field] if field is not None else None
        )

    field = data.get("sns_arn", None)
    if field is not None:
        args["sns_arn"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return Webhook(**args)


def unmarshal_CreateEmailResponse(data: Any) -> CreateEmailResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CreateEmailResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails", None)
    if field is not None:
        args["emails"] = (
            [unmarshal_Email(v) for v in field] if field is not None else None
        )

    return CreateEmailResponse(**args)


def unmarshal_DomainLastStatusDkimRecord(data: Any) -> DomainLastStatusDkimRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusDkimRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusDkimRecord(**args)


def unmarshal_DomainLastStatusDmarcRecord(data: Any) -> DomainLastStatusDmarcRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusDmarcRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusDmarcRecord(**args)


def unmarshal_DomainLastStatusSpfRecord(data: Any) -> DomainLastStatusSpfRecord:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatusSpfRecord' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("last_valid_at", None)
    if field is not None:
        args["last_valid_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["last_valid_at"] = None

    field = data.get("error", None)
    if field is not None:
        args["error"] = field
    else:
        args["error"] = None

    return DomainLastStatusSpfRecord(**args)


def unmarshal_DomainLastStatus(data: Any) -> DomainLastStatus:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DomainLastStatus' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("domain_name", None)
    if field is not None:
        args["domain_name"] = field

    field = data.get("spf_record", None)
    if field is not None:
        args["spf_record"] = unmarshal_DomainLastStatusSpfRecord(field)
    else:
        args["spf_record"] = None

    field = data.get("dkim_record", None)
    if field is not None:
        args["dkim_record"] = unmarshal_DomainLastStatusDkimRecord(field)
    else:
        args["dkim_record"] = None

    field = data.get("dmarc_record", None)
    if field is not None:
        args["dmarc_record"] = unmarshal_DomainLastStatusDmarcRecord(field)
    else:
        args["dmarc_record"] = None

    return DomainLastStatus(**args)


def unmarshal_ListDomainsResponse(data: Any) -> ListDomainsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("domains", None)
    if field is not None:
        args["domains"] = (
            [unmarshal_Domain(v) for v in field] if field is not None else None
        )

    return ListDomainsResponse(**args)


def unmarshal_ListEmailsResponse(data: Any) -> ListEmailsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListEmailsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("emails", None)
    if field is not None:
        args["emails"] = (
            [unmarshal_Email(v) for v in field] if field is not None else None
        )

    return ListEmailsResponse(**args)


def unmarshal_WebhookEvent(data: Any) -> WebhookEvent:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'WebhookEvent' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("webhook_id", None)
    if field is not None:
        args["webhook_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("domain_id", None)
    if field is not None:
        args["domain_id"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("data", None)
    if field is not None:
        args["data"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("email_id", None)
    if field is not None:
        args["email_id"] = field
    else:
        args["email_id"] = None

    return WebhookEvent(**args)


def unmarshal_ListWebhookEventsResponse(data: Any) -> ListWebhookEventsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListWebhookEventsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("webhook_events", None)
    if field is not None:
        args["webhook_events"] = (
            [unmarshal_WebhookEvent(v) for v in field] if field is not None else None
        )

    return ListWebhookEventsResponse(**args)


def unmarshal_ListWebhooksResponse(data: Any) -> ListWebhooksResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListWebhooksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("webhooks", None)
    if field is not None:
        args["webhooks"] = (
            [unmarshal_Webhook(v) for v in field] if field is not None else None
        )

    return ListWebhooksResponse(**args)


def unmarshal_Statistics(data: Any) -> Statistics:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Statistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("new_count", None)
    if field is not None:
        args["new_count"] = field

    field = data.get("sending_count", None)
    if field is not None:
        args["sending_count"] = field

    field = data.get("sent_count", None)
    if field is not None:
        args["sent_count"] = field

    field = data.get("failed_count", None)
    if field is not None:
        args["failed_count"] = field

    field = data.get("canceled_count", None)
    if field is not None:
        args["canceled_count"] = field

    return Statistics(**args)


def marshal_CreateDomainRequest(
    request: CreateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.domain_name is not None:
        output["domain_name"] = request.domain_name

    if request.accept_tos is not None:
        output["accept_tos"] = request.accept_tos

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_CreateEmailRequestAddress(
    request: CreateEmailRequestAddress,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.email is not None:
        output["email"] = request.email

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateEmailRequestAttachment(
    request: CreateEmailRequestAttachment,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.type_ is not None:
        output["type"] = request.type_

    if request.content is not None:
        output["content"] = request.content

    return output


def marshal_CreateEmailRequestHeader(
    request: CreateEmailRequestHeader,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.key is not None:
        output["key"] = request.key

    if request.value is not None:
        output["value"] = request.value

    return output


def marshal_CreateEmailRequest(
    request: CreateEmailRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.from_ is not None:
        output["from"] = marshal_CreateEmailRequestAddress(request.from_, defaults)

    if request.subject is not None:
        output["subject"] = request.subject

    if request.text is not None:
        output["text"] = request.text

    if request.html is not None:
        output["html"] = request.html

    if request.to is not None:
        output["to"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.to
        ]

    if request.cc is not None:
        output["cc"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.cc
        ]

    if request.bcc is not None:
        output["bcc"] = [
            marshal_CreateEmailRequestAddress(item, defaults) for item in request.bcc
        ]

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.attachments is not None:
        output["attachments"] = [
            marshal_CreateEmailRequestAttachment(item, defaults)
            for item in request.attachments
        ]

    if request.send_before is not None:
        output["send_before"] = request.send_before.isoformat()

    if request.additional_headers is not None:
        output["additional_headers"] = [
            marshal_CreateEmailRequestHeader(item, defaults)
            for item in request.additional_headers
        ]

    return output


def marshal_CreateWebhookRequest(
    request: CreateWebhookRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.domain_id is not None:
        output["domain_id"] = request.domain_id

    if request.name is not None:
        output["name"] = request.name

    if request.sns_arn is not None:
        output["sns_arn"] = request.sns_arn

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.event_types is not None:
        output["event_types"] = [str(item) for item in request.event_types]

    return output


def marshal_UpdateWebhookRequest(
    request: UpdateWebhookRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.event_types is not None:
        output["event_types"] = [str(item) for item in request.event_types]

    if request.sns_arn is not None:
        output["sns_arn"] = request.sns_arn

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from .types import (
    Folder,
    EphemeralProperties,
    SecretVersion,
    EphemeralPolicy,
    Secret,
    AccessSecretVersionResponse,
    ListFoldersResponse,
    ListSecretVersionsResponse,
    ListSecretsResponse,
    ListTagsResponse,
    AddSecretOwnerRequest,
    CreateFolderRequest,
    CreateSecretRequest,
    PasswordGenerationParams,
    CreateSecretVersionRequest,
    GeneratePasswordRequest,
    UpdateSecretRequest,
    UpdateSecretVersionRequest,
)


def unmarshal_Folder(data: Any) -> Folder:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Folder' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("path", None)
    if field is not None:
        args["path"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    return Folder(**args)


def unmarshal_EphemeralProperties(data: Any) -> EphemeralProperties:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'EphemeralProperties' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("action", None)
    if field is not None:
        args["action"] = field

    field = data.get("expires_at", None)
    if field is not None:
        args["expires_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["expires_at"] = None

    field = data.get("expires_once_accessed", None)
    if field is not None:
        args["expires_once_accessed"] = field
    else:
        args["expires_once_accessed"] = None

    return EphemeralProperties(**args)


def unmarshal_SecretVersion(data: Any) -> SecretVersion:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'SecretVersion' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("revision", None)
    if field is not None:
        args["revision"] = field

    field = data.get("secret_id", None)
    if field is not None:
        args["secret_id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("is_latest", None)
    if field is not None:
        args["is_latest"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("description", None)
    if field is not None:
        args["description"] = field
    else:
        args["description"] = None

    field = data.get("ephemeral_properties", None)
    if field is not None:
        args["ephemeral_properties"] = unmarshal_EphemeralProperties(field)
    else:
        args["ephemeral_properties"] = None

    return SecretVersion(**args)


def unmarshal_EphemeralPolicy(data: Any) -> EphemeralPolicy:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'EphemeralPolicy' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("action", None)
    if field is not None:
        args["action"] = field

    field = data.get("time_to_live", None)
    if field is not None:
        args["time_to_live"] = field
    else:
        args["time_to_live"] = None

    field = data.get("expires_once_accessed", None)
    if field is not None:
        args["expires_once_accessed"] = field
    else:
        args["expires_once_accessed"] = None

    return EphemeralPolicy(**args)


def unmarshal_Secret(data: Any) -> Secret:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Secret' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("version_count", None)
    if field is not None:
        args["version_count"] = field

    field = data.get("is_managed", None)
    if field is not None:
        args["is_managed"] = field

    field = data.get("is_protected", None)
    if field is not None:
        args["is_protected"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("path", None)
    if field is not None:
        args["path"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("description", None)
    if field is not None:
        args["description"] = field
    else:
        args["description"] = None

    field = data.get("ephemeral_policy", None)
    if field is not None:
        args["ephemeral_policy"] = unmarshal_EphemeralPolicy(field)
    else:
        args["ephemeral_policy"] = None

    return Secret(**args)


def unmarshal_AccessSecretVersionResponse(data: Any) -> AccessSecretVersionResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'AccessSecretVersionResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("secret_id", None)
    if field is not None:
        args["secret_id"] = field

    field = data.get("revision", None)
    if field is not None:
        args["revision"] = field

    field = data.get("data", None)
    if field is not None:
        args["data"] = field

    field = data.get("data_crc32", None)
    if field is not None:
        args["data_crc32"] = field
    else:
        args["data_crc32"] = None

    return AccessSecretVersionResponse(**args)


def unmarshal_ListFoldersResponse(data: Any) -> ListFoldersResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListFoldersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("folders", None)
    if field is not None:
        args["folders"] = (
            [unmarshal_Folder(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListFoldersResponse(**args)


def unmarshal_ListSecretVersionsResponse(data: Any) -> ListSecretVersionsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListSecretVersionsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("versions", None)
    if field is not None:
        args["versions"] = (
            [unmarshal_SecretVersion(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListSecretVersionsResponse(**args)


def unmarshal_ListSecretsResponse(data: Any) -> ListSecretsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListSecretsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("secrets", None)
    if field is not None:
        args["secrets"] = (
            [unmarshal_Secret(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListSecretsResponse(**args)


def unmarshal_ListTagsResponse(data: Any) -> ListTagsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListTagsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListTagsResponse(**args)


def marshal_AddSecretOwnerRequest(
    request: AddSecretOwnerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.product_name is not None:
        output["product_name"] = request.product_name

    if request.product is not None:
        output["product"] = str(request.product)

    return output


def marshal_CreateFolderRequest(
    request: CreateFolderRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.path is not None:
        output["path"] = request.path

    return output


def marshal_EphemeralPolicy(
    request: EphemeralPolicy,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = str(request.action)

    if request.time_to_live is not None:
        output["time_to_live"] = request.time_to_live

    if request.expires_once_accessed is not None:
        output["expires_once_accessed"] = request.expires_once_accessed

    return output


def marshal_CreateSecretRequest(
    request: CreateSecretRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.is_protected is not None:
        output["is_protected"] = request.is_protected

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.tags is not None:
        output["tags"] = request.tags

    if request.description is not None:
        output["description"] = request.description

    if request.type_ is not None:
        output["type"] = str(request.type_)

    if request.path is not None:
        output["path"] = request.path

    if request.ephemeral_policy is not None:
        output["ephemeral_policy"] = marshal_EphemeralPolicy(
            request.ephemeral_policy, defaults
        )

    return output


def marshal_PasswordGenerationParams(
    request: PasswordGenerationParams,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.length is not None:
        output["length"] = request.length

    if request.no_lowercase_letters is not None:
        output["no_lowercase_letters"] = request.no_lowercase_letters

    if request.no_uppercase_letters is not None:
        output["no_uppercase_letters"] = request.no_uppercase_letters

    if request.no_digits is not None:
        output["no_digits"] = request.no_digits

    if request.additional_chars is not None:
        output["additional_chars"] = request.additional_chars

    return output


def marshal_CreateSecretVersionRequest(
    request: CreateSecretVersionRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.data is not None:
        output["data"] = request.data

    if request.description is not None:
        output["description"] = request.description

    if request.disable_previous is not None:
        output["disable_previous"] = request.disable_previous

    if request.password_generation is not None:
        output["password_generation"] = marshal_PasswordGenerationParams(
            request.password_generation, defaults
        )

    if request.data_crc32 is not None:
        output["data_crc32"] = request.data_crc32

    return output


def marshal_GeneratePasswordRequest(
    request: GeneratePasswordRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.length is not None:
        output["length"] = request.length

    if request.description is not None:
        output["description"] = request.description

    if request.disable_previous is not None:
        output["disable_previous"] = request.disable_previous

    if request.no_lowercase_letters is not None:
        output["no_lowercase_letters"] = request.no_lowercase_letters

    if request.no_uppercase_letters is not None:
        output["no_uppercase_letters"] = request.no_uppercase_letters

    if request.no_digits is not None:
        output["no_digits"] = request.no_digits

    if request.additional_chars is not None:
        output["additional_chars"] = request.additional_chars

    return output


def marshal_UpdateSecretRequest(
    request: UpdateSecretRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.tags is not None:
        output["tags"] = request.tags

    if request.description is not None:
        output["description"] = request.description

    if request.path is not None:
        output["path"] = request.path

    if request.ephemeral_policy is not None:
        output["ephemeral_policy"] = marshal_EphemeralPolicy(
            request.ephemeral_policy, defaults
        )

    return output


def marshal_EphemeralProperties(
    request: EphemeralProperties,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = str(request.action)

    if request.expires_at is not None:
        output["expires_at"] = request.expires_at.isoformat()

    if request.expires_once_accessed is not None:
        output["expires_once_accessed"] = request.expires_once_accessed

    return output


def marshal_UpdateSecretVersionRequest(
    request: UpdateSecretVersionRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.ephemeral_properties is not None:
        output["ephemeral_properties"] = marshal_EphemeralProperties(
            request.ephemeral_properties, defaults
        )

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    TimeSeries,
    Zone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class AvailableClusterSettingPropertyType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    BOOLEAN = "boolean"
    INT = "int"
    STRING = "string"

    def __str__(self) -> str:
        return str(self.value)


class ClusterStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    READY = "ready"
    PROVISIONING = "provisioning"
    CONFIGURING = "configuring"
    DELETING = "deleting"
    ERROR = "error"
    AUTOHEALING = "autohealing"
    LOCKED = "locked"
    SUSPENDED = "suspended"
    INITIALIZING = "initializing"

    def __str__(self) -> str:
        return str(self.value)


class ListClustersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class NodeTypeStock(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN = "unknown"
    LOW_STOCK = "low_stock"
    OUT_OF_STOCK = "out_of_stock"
    AVAILABLE = "available"

    def __str__(self) -> str:
        return str(self.value)


class PrivateNetworkProvisioningMode(str, Enum, metaclass=StrEnumMeta):
    STATIC = "static"
    IPAM = "ipam"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class EndpointSpecPrivateNetworkSpecIpamConfig:
    pass


@dataclass
class PrivateNetwork:
    id: str
    """
    UUID of the Private Network.
    """

    service_ips: List[str]
    """
    List of IPv4 CIDR notation addresses of the endpoint.
    """

    zone: Zone
    """
    Zone of the Private Network.
    """

    provisioning_mode: PrivateNetworkProvisioningMode
    """
    How your endpoint ips are provisioned.
    """


@dataclass
class PublicNetwork:
    pass


@dataclass
class EndpointSpecPrivateNetworkSpec:
    id: str
    """
    UUID of the Private Network to connect to the Database Instance.
    """

    service_ips: List[str]
    """
    Endpoint IPv4 address with a CIDR notation. You must provide at least one IPv4 per node.
    """

    ipam_config: Optional[EndpointSpecPrivateNetworkSpecIpamConfig]
    """
    Automated configuration of your Private Network endpoint with Scaleway IPAM service.
    """


@dataclass
class EndpointSpecPublicNetworkSpec:
    pass


@dataclass
class AvailableClusterSetting:
    name: str
    """
    Name of the setting.
    """

    type_: AvailableClusterSettingPropertyType
    """
    Type of setting.
    """

    description: str
    """
    Description of the setting.
    """

    deprecated: bool
    """
    Defines whether or not the setting is deprecated.
    """

    default_value: Optional[str]
    """
    Default value of the setting.
    """

    max_value: Optional[int]
    """
    Optional maximum value of the setting.
    """

    min_value: Optional[int]
    """
    Optional minimum value of the setting.
    """

    regex: Optional[str]
    """
    Optional validation rule of the setting.
    """


@dataclass
class ACLRule:
    id: str
    """
    ID of the rule.
    """

    ip_cidr: Optional[str]
    """
    IPv4 network address of the rule.
    """

    description: Optional[str]
    """
    Description of the rule.
    """


@dataclass
class ClusterSetting:
    value: str
    """
    Value of the setting.
    """

    name: str
    """
    Name of the setting.
    """


@dataclass
class Endpoint:
    port: int
    """
    TCP port of the endpoint.
    """

    ips: List[str]
    """
    List of IPv4 addresses of the endpoint.
    """

    id: str
    """
    UUID of the endpoint.
    """

    private_network: Optional[PrivateNetwork]

    public_network: Optional[PublicNetwork]


@dataclass
class ACLRuleSpec:
    ip_cidr: str
    """
    IPv4 network address of the rule.
    """

    description: str
    """
    Description of the rule.
    """


@dataclass
class EndpointSpec:
    private_network: Optional[EndpointSpecPrivateNetworkSpec]

    public_network: Optional[EndpointSpecPublicNetworkSpec]


@dataclass
class ClusterVersion:
    version: str
    """
    Redis™ engine version.
    """

    available_settings: List[AvailableClusterSetting]
    """
    Cluster settings available to be updated.
    """

    logo_url: str
    """
    Redis™ logo url.
    """

    zone: Zone
    """
    Zone of the Redis™ Database Instance.
    """

    end_of_life_at: Optional[datetime]
    """
    Date of End of Life.
    """


@dataclass
class Cluster:
    id: str
    """
    UUID of the Database Instance.
    """

    name: str
    """
    Name of the Database Instance.
    """

    project_id: str
    """
    Project ID the Database Instance belongs to.
    """

    status: ClusterStatus
    """
    Status of the Database Instance.
    """

    version: str
    """
    Redis™ engine version of the Database Instance.
    """

    endpoints: List[Endpoint]
    """
    List of Database Instance endpoints.
    """

    tags: List[str]
    """
    List of tags applied to the Database Instance.
    """

    node_type: str
    """
    Node type of the Database Instance.
    """

    tls_enabled: bool
    """
    Defines whether or not TLS is enabled.
    """

    cluster_settings: List[ClusterSetting]
    """
    List of Database Instance settings.
    """

    created_at: Optional[datetime]
    """
    Creation date (Format ISO 8601).
    """

    updated_at: Optional[datetime]
    """
    Update date (Format ISO 8601).
    """

    acl_rules: List[ACLRule]
    """
    List of ACL rules.
    """

    cluster_size: int
    """
    Number of nodes of the Database Instance cluster.
    """

    zone: Zone
    """
    Zone of the Database Instance.
    """

    user_name: str
    """
    Name of the user associated to the cluster.
    """

    upgradable_versions: List[str]
    """
    List of engine versions the Database Instance can upgrade to.
    """


@dataclass
class NodeType:
    name: str
    """
    Node type name.
    """

    stock_status: NodeTypeStock
    """
    Current stock status of the node type.
    """

    description: str
    """
    Current specifications of the offer.
    """

    vcpus: int
    """
    Number of virtual CPUs.
    """

    memory: int
    """
    Quantity of RAM.
    """

    disabled: bool
    """
    Defines whether node type is currently disabled or not.
    """

    beta: bool
    """
    Defines whether node type is currently in beta.
    """

    zone: Zone
    """
    Zone of the node type.
    """


@dataclass
class AddAclRulesRequest:
    cluster_id: str
    """
    UUID of the Database Instance you want to add ACL rules to.
    """

    acl_rules: List[ACLRuleSpec]
    """
    ACLs rules to add to the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class AddAclRulesResponse:
    acl_rules: List[ACLRule]
    """
    ACL Rules enabled for the Database Instance.
    """

    total_count: int
    """
    Total count of ACL rules of the Database Instance.
    """


@dataclass
class AddClusterSettingsRequest:
    cluster_id: str
    """
    UUID of the Database Instance you want to add settings to.
    """

    settings: List[ClusterSetting]
    """
    Settings to add to the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class AddEndpointsRequest:
    cluster_id: str
    """
    UUID of the Database Instance you want to add endpoints to.
    """

    endpoints: List[EndpointSpec]
    """
    Endpoints to add to the Database Instance.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class AddEndpointsResponse:
    endpoints: List[Endpoint]
    """
    Endpoints defined on the Database Instance.
    """

    total_count: int
    """
    Total count of endpoints of the Database Instance.
    """


@dataclass
class ClusterMetricsResponse:
    timeseries: List[TimeSeries]
    """
    Time series of metrics of a given cluster.
    """


@dataclass
class ClusterSettingsResponse:
    settings: List[ClusterSetting]
    """
    Settings configured for a given Database Instance.
    """


@dataclass
class CreateClusterRequest:
    version: str
    """
    Redis™ engine version of the Database Instance.
    """

    node_type: str
    """
    Type of node to use for the Database Instance.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID in which to create the Database Instance.
    """

    name: Optional[str]
    """
    Name of the Database Instance.
    """

    tags: Optional[List[str]]
    """
    Tags to apply to the Database Instance.
    """

    user_name: str
    """
    Name of the user created upon Database Instance creation.
    """

    password: str
    """
    Password of the user.
    """

    tls_enabled: bool
    """
    Defines whether or not TLS is enabled.
    """

    cluster_size: Optional[int]
    """
    Number of nodes in the Redis™ cluster.
    """

    acl_rules: Optional[List[ACLRuleSpec]]
    """
    List of ACLRuleSpec used to secure your publicly exposed cluster.
    """

    endpoints: Optional[List[EndpointSpec]]
    """
    Zero or multiple EndpointSpec used to expose your cluster publicly and inside private networks. If no EndpoindSpec is given the cluster will be publicly exposed by default.
    """

    cluster_settings: Optional[List[ClusterSetting]]
    """
    List of advanced settings to be set upon Database Instance initialization.
    """


@dataclass
class DeleteAclRuleRequest:
    acl_id: str
    """
    UUID of the ACL rule you want to delete.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteClusterRequest:
    cluster_id: str
    """
    UUID of the Database Instance to delete.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteClusterSettingRequest:
    cluster_id: str
    """
    UUID of the Database Instance where the settings must be set.
    """

    setting_name: str
    """
    Setting name to delete.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class DeleteEndpointRequest:
    endpoint_id: str
    """
    UUID of the endpoint you want to delete.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetAclRuleRequest:
    acl_id: str
    """
    UUID of the ACL rule you want to get.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetClusterCertificateRequest:
    cluster_id: str
    """
    UUID of the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetClusterMetricsRequest:
    cluster_id: str
    """
    UUID of the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    start_at: Optional[datetime]
    """
    Start date.
    """

    end_at: Optional[datetime]
    """
    End date.
    """

    metric_name: Optional[str]
    """
    Name of the metric to gather.
    """


@dataclass
class GetClusterRequest:
    cluster_id: str
    """
    UUID of the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetEndpointRequest:
    endpoint_id: str
    """
    UUID of the endpoint you want to get.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListClusterVersionsRequest:
    include_disabled: bool
    """
    Defines whether or not to include disabled Redis™ engine versions.
    """

    include_beta: bool
    """
    Defines whether or not to include beta Redis™ engine versions.
    """

    include_deprecated: bool
    """
    Defines whether or not to include deprecated Redis™ engine versions.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    version: Optional[str]
    """
    List Redis™ engine versions that match a given name pattern.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListClusterVersionsResponse:
    versions: List[ClusterVersion]
    """
    List of available Redis™ engine versions.
    """

    total_count: int
    """
    Total count of available Redis™ engine versions.
    """


@dataclass
class ListClustersRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    tags: Optional[List[str]]
    """
    Filter by Database Instance tags.
    """

    name: Optional[str]
    """
    Filter by Database Instance names.
    """

    order_by: Optional[ListClustersRequestOrderBy]
    """
    Criteria to use when ordering the list.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    version: Optional[str]
    """
    Filter by Redis™ engine version.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListClustersResponse:
    clusters: List[Cluster]
    """
    List all Database Instances.
    """

    total_count: int
    """
    Total count of Database Instances.
    """


@dataclass
class ListNodeTypesRequest:
    include_disabled_types: bool
    """
    Defines whether or not to include disabled types.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListNodeTypesResponse:
    node_types: List[NodeType]
    """
    Types of node.
    """

    total_count: int
    """
    Total count of node types available.
    """


@dataclass
class MigrateClusterRequest:
    cluster_id: str
    """
    UUID of the Database Instance to update.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    version: Optional[str]

    node_type: Optional[str]

    cluster_size: Optional[int]


@dataclass
class RenewClusterCertificateRequest:
    cluster_id: str
    """
    UUID of the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetAclRulesRequest:
    cluster_id: str
    """
    UUID of the Database Instance where the ACL rules have to be set.
    """

    acl_rules: List[ACLRuleSpec]
    """
    ACLs rules to define for the cluster.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetAclRulesResponse:
    acl_rules: List[ACLRule]
    """
    ACL Rules enabled for the Database Instance.
    """


@dataclass
class SetClusterSettingsRequest:
    cluster_id: str
    """
    UUID of the Database Instance where the settings must be set.
    """

    settings: List[ClusterSetting]
    """
    Settings to define for the Database Instance.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetEndpointsRequest:
    cluster_id: str
    """
    UUID of the Database Instance where the endpoints have to be set.
    """

    endpoints: List[EndpointSpec]
    """
    Endpoints to define for the Database Instance.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class SetEndpointsResponse:
    endpoints: List[Endpoint]
    """
    Endpoints defined on the Database Instance.
    """


@dataclass
class UpdateClusterRequest:
    cluster_id: str
    """
    UUID of the Database Instance to update.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Name of the Database Instance.
    """

    tags: Optional[List[str]]
    """
    Database Instance tags.
    """

    user_name: Optional[str]
    """
    Name of the Database Instance user.
    """

    password: Optional[str]
    """
    Password of the Database Instance user.
    """


@dataclass
class UpdateEndpointRequest:
    endpoint_id: str
    """
    UUID of the endpoint you want to get.
    """

    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    private_network: Optional[EndpointSpecPrivateNetworkSpec]

    public_network: Optional[EndpointSpecPublicNetworkSpec]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import AvailableClusterSettingPropertyType
from .types import ClusterStatus
from .content import CLUSTER_TRANSIENT_STATUSES
from .types import ListClustersRequestOrderBy
from .types import NodeTypeStock
from .types import PrivateNetworkProvisioningMode
from .types import EndpointSpecPrivateNetworkSpecIpamConfig
from .types import PrivateNetwork
from .types import PublicNetwork
from .types import EndpointSpecPrivateNetworkSpec
from .types import EndpointSpecPublicNetworkSpec
from .types import AvailableClusterSetting
from .types import ACLRule
from .types import ClusterSetting
from .types import Endpoint
from .types import ACLRuleSpec
from .types import EndpointSpec
from .types import ClusterVersion
from .types import Cluster
from .types import NodeType
from .types import AddAclRulesRequest
from .types import AddAclRulesResponse
from .types import AddClusterSettingsRequest
from .types import AddEndpointsRequest
from .types import AddEndpointsResponse
from .types import ClusterMetricsResponse
from .types import ClusterSettingsResponse
from .types import CreateClusterRequest
from .types import DeleteAclRuleRequest
from .types import DeleteClusterRequest
from .types import DeleteClusterSettingRequest
from .types import DeleteEndpointRequest
from .types import GetAclRuleRequest
from .types import GetClusterCertificateRequest
from .types import GetClusterMetricsRequest
from .types import GetClusterRequest
from .types import GetEndpointRequest
from .types import ListClusterVersionsRequest
from .types import ListClusterVersionsResponse
from .types import ListClustersRequest
from .types import ListClustersResponse
from .types import ListNodeTypesRequest
from .types import ListNodeTypesResponse
from .types import MigrateClusterRequest
from .types import RenewClusterCertificateRequest
from .types import SetAclRulesRequest
from .types import SetAclRulesResponse
from .types import SetClusterSettingsRequest
from .types import SetEndpointsRequest
from .types import SetEndpointsResponse
from .types import UpdateClusterRequest
from .types import UpdateEndpointRequest
from .api import RedisV1API

__all__ = [
    "AvailableClusterSettingPropertyType",
    "ClusterStatus",
    "CLUSTER_TRANSIENT_STATUSES",
    "ListClustersRequestOrderBy",
    "NodeTypeStock",
    "PrivateNetworkProvisioningMode",
    "EndpointSpecPrivateNetworkSpecIpamConfig",
    "PrivateNetwork",
    "PublicNetwork",
    "EndpointSpecPrivateNetworkSpec",
    "EndpointSpecPublicNetworkSpec",
    "AvailableClusterSetting",
    "ACLRule",
    "ClusterSetting",
    "Endpoint",
    "ACLRuleSpec",
    "EndpointSpec",
    "ClusterVersion",
    "Cluster",
    "NodeType",
    "AddAclRulesRequest",
    "AddAclRulesResponse",
    "AddClusterSettingsRequest",
    "AddEndpointsRequest",
    "AddEndpointsResponse",
    "ClusterMetricsResponse",
    "ClusterSettingsResponse",
    "CreateClusterRequest",
    "DeleteAclRuleRequest",
    "DeleteClusterRequest",
    "DeleteClusterSettingRequest",
    "DeleteEndpointRequest",
    "GetAclRuleRequest",
    "GetClusterCertificateRequest",
    "GetClusterMetricsRequest",
    "GetClusterRequest",
    "GetEndpointRequest",
    "ListClusterVersionsRequest",
    "ListClusterVersionsResponse",
    "ListClustersRequest",
    "ListClustersResponse",
    "ListNodeTypesRequest",
    "ListNodeTypesResponse",
    "MigrateClusterRequest",
    "RenewClusterCertificateRequest",
    "SetAclRulesRequest",
    "SetAclRulesResponse",
    "SetClusterSettingsRequest",
    "SetEndpointsRequest",
    "SetEndpointsResponse",
    "UpdateClusterRequest",
    "UpdateEndpointRequest",
    "RedisV1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.utils import (
    OneOfPossibility,
    WaitForOptions,
    resolve_one_of,
    validate_path_param,
    fetch_all_pages,
    wait_for_resource,
)
from .types import (
    ApplicationType,
    ListApplicationsRequestOrderBy,
    ListJobResultsRequestOrderBy,
    ListJobsRequestOrderBy,
    ListPlatformsRequestOrderBy,
    ListProcessResultsRequestOrderBy,
    ListProcessesRequestOrderBy,
    ListSessionACLsRequestOrderBy,
    ListSessionsRequestOrderBy,
    PlatformTechnology,
    PlatformType,
    SessionAccess,
    Application,
    CreateJobRequest,
    CreateProcessRequest,
    CreateSessionRequest,
    Job,
    JobCircuit,
    JobResult,
    ListApplicationsResponse,
    ListJobResultsResponse,
    ListJobsResponse,
    ListPlatformsResponse,
    ListProcessResultsResponse,
    ListProcessesResponse,
    ListSessionACLsResponse,
    ListSessionsResponse,
    Platform,
    Process,
    ProcessResult,
    Session,
    UpdateJobRequest,
    UpdateProcessRequest,
    UpdateSessionRequest,
)
from .content import (
    JOB_TRANSIENT_STATUSES,
    PROCESS_TRANSIENT_STATUSES,
    SESSION_TRANSIENT_STATUSES,
)
from .marshalling import (
    unmarshal_JobCircuit,
    unmarshal_Application,
    unmarshal_Job,
    unmarshal_Platform,
    unmarshal_Process,
    unmarshal_Session,
    unmarshal_ListApplicationsResponse,
    unmarshal_ListJobResultsResponse,
    unmarshal_ListJobsResponse,
    unmarshal_ListPlatformsResponse,
    unmarshal_ListProcessResultsResponse,
    unmarshal_ListProcessesResponse,
    unmarshal_ListSessionACLsResponse,
    unmarshal_ListSessionsResponse,
    marshal_CreateJobRequest,
    marshal_CreateProcessRequest,
    marshal_CreateSessionRequest,
    marshal_UpdateJobRequest,
    marshal_UpdateProcessRequest,
    marshal_UpdateSessionRequest,
)


class QaasV1Alpha1API(API):
    """
    This API allows you to manage Scaleway Quantum as a Service.
    """

    def get_job(
        self,
        *,
        job_id: str,
    ) -> Job:
        """
        Get job information.
        Retrieve information about the provided **job ID**, such as status, payload, and result.
        :param job_id: Unique ID of the job you want to get.
        :return: :class:`Job <Job>`

        Usage:
        ::

            result = api.get_job(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/jobs/{param_job_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Job(res.json())

    def wait_for_job(
        self,
        *,
        job_id: str,
        options: Optional[WaitForOptions[Job, bool]] = None,
    ) -> Job:
        """
        Get job information.
        Retrieve information about the provided **job ID**, such as status, payload, and result.
        :param job_id: Unique ID of the job you want to get.
        :return: :class:`Job <Job>`

        Usage:
        ::

            result = api.get_job(
                job_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in JOB_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_job,
            options=options,
            args={
                "job_id": job_id,
            },
        )

    def list_jobs(
        self,
        *,
        session_id: Optional[str] = None,
        project_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListJobsRequestOrderBy] = None,
    ) -> ListJobsResponse:
        """
        List all jobs within a project or session.
        Retrieve information about all jobs within a given project or session.
        :param session_id: List jobs with this session ID.
        One-Of ('resource_id'): at most one of 'session_id', 'project_id' could be set.
        :param project_id: List jobs with this project ID.
        One-Of ('resource_id'): at most one of 'session_id', 'project_id' could be set.
        :param tags: List jobs with these tags.
        :param page: Page number.
        :param page_size: Maximum number of jobs to return per page.
        :param order_by: Sort order of the returned jobs.
        :return: :class:`ListJobsResponse <ListJobsResponse>`

        Usage:
        ::

            result = api.list_jobs()
        """

        res = self._request(
            "GET",
            "/qaas/v1alpha1/jobs",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "tags": tags,
                **resolve_one_of(
                    [
                        OneOfPossibility("project_id", project_id),
                        OneOfPossibility("session_id", session_id),
                    ]
                ),
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListJobsResponse(res.json())

    def list_jobs_all(
        self,
        *,
        session_id: Optional[str] = None,
        project_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListJobsRequestOrderBy] = None,
    ) -> List[Job]:
        """
        List all jobs within a project or session.
        Retrieve information about all jobs within a given project or session.
        :param session_id: List jobs with this session ID.
        One-Of ('resource_id'): at most one of 'session_id', 'project_id' could be set.
        :param project_id: List jobs with this project ID.
        One-Of ('resource_id'): at most one of 'session_id', 'project_id' could be set.
        :param tags: List jobs with these tags.
        :param page: Page number.
        :param page_size: Maximum number of jobs to return per page.
        :param order_by: Sort order of the returned jobs.
        :return: :class:`List[Job] <List[Job]>`

        Usage:
        ::

            result = api.list_jobs_all()
        """

        return fetch_all_pages(
            type=ListJobsResponse,
            key="jobs",
            fetcher=self.list_jobs,
            args={
                "tags": tags,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "session_id": session_id,
                "project_id": project_id,
            },
        )

    def list_job_results(
        self,
        *,
        job_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListJobResultsRequestOrderBy] = None,
    ) -> ListJobResultsResponse:
        """
        List all results of a job.
        Retrieve all intermediate and final results of a job.
        :param job_id: ID of the job.
        :param page: Page number.
        :param page_size: Maximum number of results to return per page.
        :param order_by: Sort order of the returned results.
        :return: :class:`ListJobResultsResponse <ListJobResultsResponse>`

        Usage:
        ::

            result = api.list_job_results(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/jobs/{param_job_id}/results",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListJobResultsResponse(res.json())

    def list_job_results_all(
        self,
        *,
        job_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListJobResultsRequestOrderBy] = None,
    ) -> List[JobResult]:
        """
        List all results of a job.
        Retrieve all intermediate and final results of a job.
        :param job_id: ID of the job.
        :param page: Page number.
        :param page_size: Maximum number of results to return per page.
        :param order_by: Sort order of the returned results.
        :return: :class:`List[JobResult] <List[JobResult]>`

        Usage:
        ::

            result = api.list_job_results_all(
                job_id="example",
            )
        """

        return fetch_all_pages(
            type=ListJobResultsResponse,
            key="job_results",
            fetcher=self.list_job_results,
            args={
                "job_id": job_id,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    def create_job(
        self,
        *,
        name: str,
        session_id: str,
        circuit: JobCircuit,
        tags: Optional[List[str]] = None,
        max_duration: Optional[str] = None,
    ) -> Job:
        """
        Create a job.
        Create a job to be executed inside a session.
        :param name: Name of the job.
        :param session_id: Session in which the job is executed.
        :param circuit: Quantum circuit that should be executed.
        :param tags: Tags of the job.
        :param max_duration: Maximum duration of the job.
        :return: :class:`Job <Job>`

        Usage:
        ::

            result = api.create_job(
                name="example",
                session_id="example",
                circuit=JobCircuit(),
            )
        """

        res = self._request(
            "POST",
            "/qaas/v1alpha1/jobs",
            body=marshal_CreateJobRequest(
                CreateJobRequest(
                    name=name,
                    session_id=session_id,
                    circuit=circuit,
                    tags=tags,
                    max_duration=max_duration,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Job(res.json())

    def update_job(
        self,
        *,
        job_id: str,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Job:
        """
        Update job information.
        Update job information about the provided **job ID**.
        :param job_id: Unique ID of the job.
        :param name: Name of the job.
        :param tags: Tags of the job.
        :return: :class:`Job <Job>`

        Usage:
        ::

            result = api.update_job(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "PATCH",
            f"/qaas/v1alpha1/jobs/{param_job_id}",
            body=marshal_UpdateJobRequest(
                UpdateJobRequest(
                    job_id=job_id,
                    name=name,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Job(res.json())

    def cancel_job(
        self,
        *,
        job_id: str,
    ) -> Job:
        """
        Cancel a job.
        Cancel the job corresponding to the provided **job ID**.
        :param job_id: Unique ID of the job.
        :return: :class:`Job <Job>`

        Usage:
        ::

            result = api.cancel_job(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "POST",
            f"/qaas/v1alpha1/jobs/{param_job_id}/cancel",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Job(res.json())

    def delete_job(
        self,
        *,
        job_id: str,
    ) -> None:
        """
        Delete a job.
        Delete the job corresponding to the provided **job ID**.
        :param job_id: Unique ID of the job.

        Usage:
        ::

            result = api.delete_job(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "DELETE",
            f"/qaas/v1alpha1/jobs/{param_job_id}",
        )

        self._throw_on_error(res)

    def get_job_circuit(
        self,
        *,
        job_id: str,
    ) -> JobCircuit:
        """
        Get a job circuit.
        Retrieve the circuit of the provided **job ID**.
        :param job_id: Unique ID of the job.
        :return: :class:`JobCircuit <JobCircuit>`

        Usage:
        ::

            result = api.get_job_circuit(
                job_id="example",
            )
        """

        param_job_id = validate_path_param("job_id", job_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/jobs/{param_job_id}/circuit",
        )

        self._throw_on_error(res)
        return unmarshal_JobCircuit(res.json())

    def get_platform(
        self,
        *,
        platform_id: str,
    ) -> Platform:
        """
        Get platform information.
        Retrieve information about the provided **platform ID**, such as provider name, technology, and type.
        :param platform_id: Unique ID of the platform.
        :return: :class:`Platform <Platform>`

        Usage:
        ::

            result = api.get_platform(
                platform_id="example",
            )
        """

        param_platform_id = validate_path_param("platform_id", platform_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/platforms/{param_platform_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Platform(res.json())

    def list_platforms(
        self,
        *,
        provider_name: Optional[str] = None,
        name: Optional[str] = None,
        platform_type: Optional[PlatformType] = None,
        platform_technology: Optional[PlatformTechnology] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListPlatformsRequestOrderBy] = None,
    ) -> ListPlatformsResponse:
        """
        List all available platforms.
        Retrieve information about all platforms.
        :param provider_name: List platforms with this provider name.
        :param name: List platforms with this name.
        :param platform_type: List platforms with this type.
        :param platform_technology: List platforms with this technology.
        :param page: Page number.
        :param page_size: Maximum number of platforms to return per page.
        :param order_by: Sort order of the returned platforms.
        :return: :class:`ListPlatformsResponse <ListPlatformsResponse>`

        Usage:
        ::

            result = api.list_platforms()
        """

        res = self._request(
            "GET",
            "/qaas/v1alpha1/platforms",
            params={
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "platform_technology": platform_technology,
                "platform_type": platform_type,
                "provider_name": provider_name,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListPlatformsResponse(res.json())

    def list_platforms_all(
        self,
        *,
        provider_name: Optional[str] = None,
        name: Optional[str] = None,
        platform_type: Optional[PlatformType] = None,
        platform_technology: Optional[PlatformTechnology] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListPlatformsRequestOrderBy] = None,
    ) -> List[Platform]:
        """
        List all available platforms.
        Retrieve information about all platforms.
        :param provider_name: List platforms with this provider name.
        :param name: List platforms with this name.
        :param platform_type: List platforms with this type.
        :param platform_technology: List platforms with this technology.
        :param page: Page number.
        :param page_size: Maximum number of platforms to return per page.
        :param order_by: Sort order of the returned platforms.
        :return: :class:`List[Platform] <List[Platform]>`

        Usage:
        ::

            result = api.list_platforms_all()
        """

        return fetch_all_pages(
            type=ListPlatformsResponse,
            key="platforms",
            fetcher=self.list_platforms,
            args={
                "provider_name": provider_name,
                "name": name,
                "platform_type": platform_type,
                "platform_technology": platform_technology,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    def get_session(
        self,
        *,
        session_id: str,
    ) -> Session:
        """
        Get session infrormation.
        Retrieve information about the provided **session ID**, such as name, status, and number of executed jobs.
        :param session_id: Unique ID of the session.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.get_session(
                session_id="example",
            )
        """

        param_session_id = validate_path_param("session_id", session_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/sessions/{param_session_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Session(res.json())

    def wait_for_session(
        self,
        *,
        session_id: str,
        options: Optional[WaitForOptions[Session, bool]] = None,
    ) -> Session:
        """
        Get session infrormation.
        Retrieve information about the provided **session ID**, such as name, status, and number of executed jobs.
        :param session_id: Unique ID of the session.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.get_session(
                session_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in SESSION_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_session,
            options=options,
            args={
                "session_id": session_id,
            },
        )

    def list_sessions(
        self,
        *,
        platform_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListSessionsRequestOrderBy] = None,
        project_id: Optional[str] = None,
    ) -> ListSessionsResponse:
        """
        List all sessions.
        Retrieve information about all sessions.
        :param platform_id: List sessions that have been created for this platform.
        :param tags: List sessions with these tags.
        :param page: Page number.
        :param page_size: Maximum number of sessions to return per page.
        :param order_by: Sort order of the returned sessions.
        :param project_id: List sessions belonging to this project ID.
        :return: :class:`ListSessionsResponse <ListSessionsResponse>`

        Usage:
        ::

            result = api.list_sessions()
        """

        res = self._request(
            "GET",
            "/qaas/v1alpha1/sessions",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "platform_id": platform_id,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListSessionsResponse(res.json())

    def list_sessions_all(
        self,
        *,
        platform_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListSessionsRequestOrderBy] = None,
        project_id: Optional[str] = None,
    ) -> List[Session]:
        """
        List all sessions.
        Retrieve information about all sessions.
        :param platform_id: List sessions that have been created for this platform.
        :param tags: List sessions with these tags.
        :param page: Page number.
        :param page_size: Maximum number of sessions to return per page.
        :param order_by: Sort order of the returned sessions.
        :param project_id: List sessions belonging to this project ID.
        :return: :class:`List[Session] <List[Session]>`

        Usage:
        ::

            result = api.list_sessions_all()
        """

        return fetch_all_pages(
            type=ListSessionsResponse,
            key="sessions",
            fetcher=self.list_sessions,
            args={
                "platform_id": platform_id,
                "tags": tags,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "project_id": project_id,
            },
        )

    def create_session(
        self,
        *,
        platform_id: str,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
        max_idle_duration: Optional[str] = None,
        max_duration: Optional[str] = None,
        tags: Optional[List[str]] = None,
        deduplication_id: Optional[str] = None,
    ) -> Session:
        """
        Create a session.
        Create a dedicated session for the specified platform.
        :param platform_id: ID of the Platform for which the session was created.
        :param project_id: ID of the Project in which the session was created.
        :param name: Name of the session.
        :param max_idle_duration: Maximum idle duration before the session ends.
        :param max_duration: Maximum duration before the session ends.
        :param tags: Tags of the session.
        :param deduplication_id: Deduplication ID of the session.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.create_session(
                platform_id="example",
            )
        """

        res = self._request(
            "POST",
            "/qaas/v1alpha1/sessions",
            body=marshal_CreateSessionRequest(
                CreateSessionRequest(
                    platform_id=platform_id,
                    project_id=project_id,
                    name=name,
                    max_idle_duration=max_idle_duration,
                    max_duration=max_duration,
                    tags=tags,
                    deduplication_id=deduplication_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Session(res.json())

    def update_session(
        self,
        *,
        session_id: str,
        name: Optional[str] = None,
        max_idle_duration: Optional[str] = None,
        max_duration: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Session:
        """
        Update session information.
        Update session information of the provided **session ID**.
        :param session_id: Unique ID of the session.
        :param name: Name of the session.
        :param max_idle_duration: Maximum idle duration before the session ends.
        :param max_duration: Maximum time before the session ends.
        :param tags: Tags of the session.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.update_session(
                session_id="example",
            )
        """

        param_session_id = validate_path_param("session_id", session_id)

        res = self._request(
            "PATCH",
            f"/qaas/v1alpha1/sessions/{param_session_id}",
            body=marshal_UpdateSessionRequest(
                UpdateSessionRequest(
                    session_id=session_id,
                    name=name,
                    max_idle_duration=max_idle_duration,
                    max_duration=max_duration,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Session(res.json())

    def terminate_session(
        self,
        *,
        session_id: str,
    ) -> Session:
        """
        Terminate an existing session.
        Terminate a session by its unique ID and cancel all its attached jobs.
        :param session_id: Unique ID of the session.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.terminate_session(
                session_id="example",
            )
        """

        param_session_id = validate_path_param("session_id", session_id)

        res = self._request(
            "POST",
            f"/qaas/v1alpha1/sessions/{param_session_id}/terminate",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Session(res.json())

    def delete_session(
        self,
        *,
        session_id: str,
    ) -> None:
        """
        Delete an existing session.
        Delete a session by its unique ID and delete all its attached jobs.
        :param session_id: Unique ID of the session.

        Usage:
        ::

            result = api.delete_session(
                session_id="example",
            )
        """

        param_session_id = validate_path_param("session_id", session_id)

        res = self._request(
            "DELETE",
            f"/qaas/v1alpha1/sessions/{param_session_id}",
        )

        self._throw_on_error(res)

    def list_session_ac_ls(
        self,
        *,
        session_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListSessionACLsRequestOrderBy] = None,
    ) -> ListSessionACLsResponse:
        """
        :param session_id:
        :param page:
        :param page_size:
        :param order_by:
        :return: :class:`ListSessionACLsResponse <ListSessionACLsResponse>`

        Usage:
        ::

            result = api.list_session_ac_ls(
                session_id="example",
            )
        """

        param_session_id = validate_path_param("session_id", session_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/sessions/{param_session_id}/acls",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListSessionACLsResponse(res.json())

    def list_session_ac_ls_all(
        self,
        *,
        session_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListSessionACLsRequestOrderBy] = None,
    ) -> List[SessionAccess]:
        """
        :param session_id:
        :param page:
        :param page_size:
        :param order_by:
        :return: :class:`List[SessionAccess] <List[SessionAccess]>`

        Usage:
        ::

            result = api.list_session_ac_ls_all(
                session_id="example",
            )
        """

        return fetch_all_pages(
            type=ListSessionACLsResponse,
            key="acls",
            fetcher=self.list_session_ac_ls,
            args={
                "session_id": session_id,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    def create_process(
        self,
        *,
        name: str,
        project_id: Optional[str] = None,
        platform_id: Optional[str] = None,
        application_id: Optional[str] = None,
        input: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Process:
        """
        Create a process.
        Create a new process for the specified application on a specified platform.
        :param name: Name of the process.
        :param project_id: ID of the project in which the process was created.
        :param platform_id: ID of the platform for which the process was created.
        :param application_id: ID of the application for which the process was created.
        :param input: Process parameters in JSON format.
        :param tags: Tags of the process.
        :return: :class:`Process <Process>`

        Usage:
        ::

            result = api.create_process(
                name="example",
            )
        """

        res = self._request(
            "POST",
            "/qaas/v1alpha1/processes",
            body=marshal_CreateProcessRequest(
                CreateProcessRequest(
                    name=name,
                    project_id=project_id,
                    platform_id=platform_id,
                    application_id=application_id,
                    input=input,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Process(res.json())

    def get_process(
        self,
        *,
        process_id: str,
    ) -> Process:
        """
        Get process infrormation.
        Retrieve information about the provided **process ID**, such as name, status and progress.
        :param process_id: Unique ID of the process.
        :return: :class:`Process <Process>`

        Usage:
        ::

            result = api.get_process(
                process_id="example",
            )
        """

        param_process_id = validate_path_param("process_id", process_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/processes/{param_process_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Process(res.json())

    def wait_for_process(
        self,
        *,
        process_id: str,
        options: Optional[WaitForOptions[Process, bool]] = None,
    ) -> Process:
        """
        Get process infrormation.
        Retrieve information about the provided **process ID**, such as name, status and progress.
        :param process_id: Unique ID of the process.
        :return: :class:`Process <Process>`

        Usage:
        ::

            result = api.get_process(
                process_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in PROCESS_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_process,
            options=options,
            args={
                "process_id": process_id,
            },
        )

    def list_processes(
        self,
        *,
        application_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProcessesRequestOrderBy] = None,
        project_id: Optional[str] = None,
    ) -> ListProcessesResponse:
        """
        List all processes.
        Retrieve information about all processes.
        :param application_id: List processes that have been created for this application.
        :param tags: List processes with these tags.
        :param page: Page number.
        :param page_size: Maximum number of processes to return per page.
        :param order_by: Sort order of the returned processes.
        :param project_id: List processes belonging to this project ID.
        :return: :class:`ListProcessesResponse <ListProcessesResponse>`

        Usage:
        ::

            result = api.list_processes()
        """

        res = self._request(
            "GET",
            "/qaas/v1alpha1/processes",
            params={
                "application_id": application_id,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListProcessesResponse(res.json())

    def list_processes_all(
        self,
        *,
        application_id: Optional[str] = None,
        tags: Optional[List[str]] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProcessesRequestOrderBy] = None,
        project_id: Optional[str] = None,
    ) -> List[Process]:
        """
        List all processes.
        Retrieve information about all processes.
        :param application_id: List processes that have been created for this application.
        :param tags: List processes with these tags.
        :param page: Page number.
        :param page_size: Maximum number of processes to return per page.
        :param order_by: Sort order of the returned processes.
        :param project_id: List processes belonging to this project ID.
        :return: :class:`List[Process] <List[Process]>`

        Usage:
        ::

            result = api.list_processes_all()
        """

        return fetch_all_pages(
            type=ListProcessesResponse,
            key="processes",
            fetcher=self.list_processes,
            args={
                "application_id": application_id,
                "tags": tags,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "project_id": project_id,
            },
        )

    def update_process(
        self,
        *,
        process_id: str,
        name: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> Process:
        """
        Update process information.
        Update process information of the provided **process ID**.
        :param process_id: Unique ID of the process.
        :param name: Name of the process.
        :param tags: Tags of the process.
        :return: :class:`Process <Process>`

        Usage:
        ::

            result = api.update_process(
                process_id="example",
            )
        """

        param_process_id = validate_path_param("process_id", process_id)

        res = self._request(
            "PATCH",
            f"/qaas/v1alpha1/processes/{param_process_id}",
            body=marshal_UpdateProcessRequest(
                UpdateProcessRequest(
                    process_id=process_id,
                    name=name,
                    tags=tags,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Process(res.json())

    def cancel_process(
        self,
        *,
        process_id: str,
    ) -> Process:
        """
        Cancel a running process.
        Cancel a process by its unique ID. Intermediate results are still available.
        :param process_id: Unique ID of the process.
        :return: :class:`Process <Process>`

        Usage:
        ::

            result = api.cancel_process(
                process_id="example",
            )
        """

        param_process_id = validate_path_param("process_id", process_id)

        res = self._request(
            "POST",
            f"/qaas/v1alpha1/processes/{param_process_id}/cancel",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Process(res.json())

    def delete_process(
        self,
        *,
        process_id: str,
    ) -> None:
        """
        Delete an existing process.
        Delete a process by its unique ID and delete all its data.
        :param process_id: Unique ID of the process.

        Usage:
        ::

            result = api.delete_process(
                process_id="example",
            )
        """

        param_process_id = validate_path_param("process_id", process_id)

        res = self._request(
            "DELETE",
            f"/qaas/v1alpha1/processes/{param_process_id}",
        )

        self._throw_on_error(res)

    def list_process_results(
        self,
        *,
        process_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProcessResultsRequestOrderBy] = None,
    ) -> ListProcessResultsResponse:
        """
        List all results of a process.
        Retrieve all intermediate and final result of a process.
        :param process_id: ID of the process.
        :param page: Page number.
        :param page_size: Maximum number of results to return per page.
        :param order_by: Sort order of the returned results.
        :return: :class:`ListProcessResultsResponse <ListProcessResultsResponse>`

        Usage:
        ::

            result = api.list_process_results(
                process_id="example",
            )
        """

        param_process_id = validate_path_param("process_id", process_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/processes/{param_process_id}/results",
            params={
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListProcessResultsResponse(res.json())

    def list_process_results_all(
        self,
        *,
        process_id: str,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListProcessResultsRequestOrderBy] = None,
    ) -> List[ProcessResult]:
        """
        List all results of a process.
        Retrieve all intermediate and final result of a process.
        :param process_id: ID of the process.
        :param page: Page number.
        :param page_size: Maximum number of results to return per page.
        :param order_by: Sort order of the returned results.
        :return: :class:`List[ProcessResult] <List[ProcessResult]>`

        Usage:
        ::

            result = api.list_process_results_all(
                process_id="example",
            )
        """

        return fetch_all_pages(
            type=ListProcessResultsResponse,
            key="process_results",
            fetcher=self.list_process_results,
            args={
                "process_id": process_id,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )

    def get_application(
        self,
        *,
        application_id: str,
    ) -> Application:
        """
        Get application information.
        Retrieve information about the provided **applcation ID**, such as name, type and compatible platforms.
        :param application_id: Unique ID of the application.
        :return: :class:`Application <Application>`

        Usage:
        ::

            result = api.get_application(
                application_id="example",
            )
        """

        param_application_id = validate_path_param("application_id", application_id)

        res = self._request(
            "GET",
            f"/qaas/v1alpha1/applications/{param_application_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Application(res.json())

    def list_applications(
        self,
        *,
        name: Optional[str] = None,
        application_type: Optional[ApplicationType] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListApplicationsRequestOrderBy] = None,
    ) -> ListApplicationsResponse:
        """
        List all available applications.
        Retrieve information about all applications.
        :param name: List applications with this name.
        :param application_type: List applications with this type.
        :param page: Page number.
        :param page_size: Maximum number of applications a to return per page.
        :param order_by: Sort order of the returned applications.
        :return: :class:`ListApplicationsResponse <ListApplicationsResponse>`

        Usage:
        ::

            result = api.list_applications()
        """

        res = self._request(
            "GET",
            "/qaas/v1alpha1/applications",
            params={
                "application_type": application_type,
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListApplicationsResponse(res.json())

    def list_applications_all(
        self,
        *,
        name: Optional[str] = None,
        application_type: Optional[ApplicationType] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListApplicationsRequestOrderBy] = None,
    ) -> List[Application]:
        """
        List all available applications.
        Retrieve information about all applications.
        :param name: List applications with this name.
        :param application_type: List applications with this type.
        :param page: Page number.
        :param page_size: Maximum number of applications a to return per page.
        :param order_by: Sort order of the returned applications.
        :return: :class:`List[Application] <List[Application]>`

        Usage:
        ::

            result = api.list_applications_all()
        """

        return fetch_all_pages(
            type=ListApplicationsResponse,
            key="applications",
            fetcher=self.list_applications,
            args={
                "name": name,
                "application_type": application_type,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
            },
        )



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class BrowseSecretsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class EphemeralPolicyAction(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_ACTION = "unknown_action"
    DELETE = "delete"
    DISABLE = "disable"

    def __str__(self) -> str:
        return str(self.value)


class ListSecretsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class Product(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_PRODUCT = "unknown_product"
    EDGE_SERVICES = "edge_services"

    def __str__(self) -> str:
        return str(self.value)


class SecretStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    READY = "ready"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class SecretType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    OPAQUE = "opaque"
    CERTIFICATE = "certificate"
    KEY_VALUE = "key_value"
    BASIC_CREDENTIALS = "basic_credentials"
    DATABASE_CREDENTIALS = "database_credentials"
    SSH_KEY = "ssh_key"

    def __str__(self) -> str:
        return str(self.value)


class SecretVersionStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    ENABLED = "enabled"
    DISABLED = "disabled"
    DELETED = "deleted"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class EphemeralPolicy:
    action: EphemeralPolicyAction
    """
    See the `EphemeralPolicy.Action` enum for a description of values.
    """

    time_to_live: Optional[str]
    """
    Time frame, from one second and up to one year, during which the secret's versions are valid.
    """

    expires_once_accessed: Optional[bool]
    """
    Returns `true` if the version expires after a single user access.
    """


@dataclass
class BrowseSecretsResponseItemFolderDetails:
    pass


@dataclass
class BrowseSecretsResponseItemSecretDetails:
    id: str

    tags: List[str]

    version_count: int

    protected: bool

    type_: SecretType

    ephemeral_policy: Optional[EphemeralPolicy]


@dataclass
class EphemeralProperties:
    action: EphemeralPolicyAction
    """
    See `EphemeralPolicy.Action` enum for a description of values.
    """

    expires_at: Optional[datetime]
    """
    (Optional.) If not specified, the version does not have an expiration date.
    """

    expires_once_accessed: Optional[bool]
    """
    (Optional.) If not specified, the version can be accessed an unlimited amount of times.
    """


@dataclass
class BrowseSecretsResponseItem:
    name: str

    created_at: Optional[datetime]

    updated_at: Optional[datetime]

    secret: Optional[BrowseSecretsResponseItemSecretDetails]

    folder: Optional[BrowseSecretsResponseItemFolderDetails]


@dataclass
class SecretVersion:
    revision: int
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1.
    """

    secret_id: str
    """
    ID of the secret.
    """

    status: SecretVersionStatus
    """
    * `unknown_status`: the version is in an invalid state.
* `enabled`: the version is accessible.
* `disabled`: the version is not accessible but can be enabled.
* `deleted`: the version is permanently deleted. It is not possible to recover it.
    """

    latest: bool
    """
    Returns `true` if the version is the latest.
    """

    created_at: Optional[datetime]
    """
    Date and time of the version's creation.
    """

    updated_at: Optional[datetime]
    """
    Last update of the version.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    ephemeral_properties: Optional[EphemeralProperties]
    """
    Returns the version's expiration date, whether it expires after being accessed once, and the action to perform (disable or delete) once the version expires.
    """


@dataclass
class Secret:
    id: str
    """
    ID of the secret.
    """

    project_id: str
    """
    ID of the Project containing the secret.
    """

    name: str
    """
    Name of the secret.
    """

    status: SecretStatus
    """
    * `ready`: the secret can be read, modified and deleted.
* `locked`: no action can be performed on the secret. This status can only be applied and removed by Scaleway.
    """

    created_at: Optional[datetime]
    """
    Date and time of the secret's creation.
    """

    updated_at: Optional[datetime]
    """
    Last update of the secret.
    """

    tags: List[str]
    """
    List of the secret's tags.
    """

    version_count: int
    """
    Number of versions for this secret.
    """

    managed: bool
    """
    Returns `true` for secrets that are managed by another product.
    """

    protected: bool
    """
    Returns `true` for protected secrets that cannot be deleted.
    """

    type_: SecretType
    """
    See the `Secret.Type` enum for a description of values.
    """

    path: str
    """
    Location of the secret in the directory structure.
    """

    region: Region
    """
    Region of the secret.
    """

    description: Optional[str]
    """
    Updated description of the secret.
    """

    ephemeral_policy: Optional[EphemeralPolicy]
    """
    (Optional.) Policy that defines whether/when a secret's versions expire. By default, the policy is applied to all the secret's versions.
    """


@dataclass
class AccessSecretVersionByPathRequest:
    secret_path: str
    """
    Secret's path.
    """

    secret_name: str
    """
    Secret's name.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- an integer (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    """


@dataclass
class AccessSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class AccessSecretVersionResponse:
    secret_id: str
    """
    ID of the secret.
    """

    revision: int
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1.
    """

    data: str
    """
    The base64-encoded secret payload of the version.
    """

    type_: SecretType
    """
    See the `Secret.Type` enum for a description of values.
    """

    data_crc32: Optional[int]
    """
    This field is only available if a CRC32 was supplied during the creation of the version.
    """


@dataclass
class AddSecretOwnerRequest:
    secret_id: str
    """
    ID of the secret.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    product: Optional[Product]
    """
    See `Product` enum for description of values.
    """


@dataclass
class BasicCredentials:
    username: str
    """
    The username or identifier associated with the credentials.
    """

    password: str
    """
    The password associated with the credentials.
    """


@dataclass
class BrowseSecretsRequest:
    prefix: str
    """
    Filter secrets and folders for a given prefix (default /).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    Filter by Project ID (optional).
    """

    order_by: Optional[BrowseSecretsRequestOrderBy]

    page: Optional[int]

    page_size: Optional[int]

    tags: Optional[List[str]]
    """
    Filter secrets by tags.
    """

    type_: Optional[SecretType]
    """
    Filter by secret type (optional).
    """


@dataclass
class BrowseSecretsResponse:
    items: List[BrowseSecretsResponseItem]
    """
    Repeated item of type secret or folder, sorted by the request parameter.
    """

    current_path: str
    """
    Current path for the given prefix.
    """

    total_count: int
    """
    Count of all secrets and folders matching the requested criteria.
    """


@dataclass
class CreateSecretRequest:
    name: str
    """
    Name of the secret.
    """

    protected: bool
    """
    A protected secret cannot be deleted.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project containing the secret.
    """

    tags: Optional[List[str]]
    """
    List of the secret's tags.
    """

    description: Optional[str]
    """
    Description of the secret.
    """

    type_: Optional[SecretType]
    """
    (Optional.) See the `Secret.Type` enum for a description of values. If not specified, the type is `Opaque`.
    """

    path: Optional[str]
    """
    (Optional.) Location of the secret in the directory structure. If not specified, the path is `/`.
    """

    ephemeral_policy: Optional[EphemeralPolicy]
    """
    (Optional.) Policy that defines whether/when a secret's versions expire. By default, the policy is applied to all the secret's versions.
    """


@dataclass
class CreateSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    data: str
    """
    The base64-encoded secret payload of the version.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    disable_previous: Optional[bool]
    """
    (Optional.) If there is no previous version or if the previous version was already disabled, does nothing.
    """

    data_crc32: Optional[int]
    """
    If specified, Secret Manager will verify the integrity of the data received against the given CRC32 checksum. An error is returned if the CRC32 does not match. If, however, the CRC32 matches, it will be stored and returned along with the SecretVersion on future access requests.
    """


@dataclass
class DatabaseCredentials:
    engine: str
    """
    Supported database engines are: 'postgres', 'mysql', 'other'.
    """

    username: str
    """
    The username used to authenticate to the database server.
    """

    password: str
    """
    The password used to authenticate to the database server.
    """

    host: str
    """
    The hostname or resolvable DNS name of the database server.
    """

    dbname: str
    """
    The name of the database to connect to.
    """

    port: str
    """
    The port must be an integer ranging from 0 to 65535.
    """


@dataclass
class DeleteSecretRequest:
    secret_id: str
    """
    ID of the secret.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DisableSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class EnableSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetSecretRequest:
    secret_id: str
    """
    ID of the secret.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class ListSecretTypesRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListSecretTypesResponse:
    types: List[SecretType]
    """
    List of secret types.
    """

    total_count: int
    """
    Count of all secret types matching the requested criteria.
    """


@dataclass
class ListSecretVersionsRequest:
    secret_id: str
    """
    ID of the secret.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]

    page_size: Optional[int]

    status: Optional[List[SecretVersionStatus]]
    """
    Filter results by status.
    """


@dataclass
class ListSecretVersionsResponse:
    versions: List[SecretVersion]
    """
    Single page of versions.
    """

    total_count: int
    """
    Number of versions.
    """


@dataclass
class ListSecretsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID (optional).
    """

    project_id: Optional[str]
    """
    Filter by Project ID (optional).
    """

    order_by: Optional[ListSecretsRequestOrderBy]

    page: Optional[int]

    page_size: Optional[int]

    tags: Optional[List[str]]
    """
    List of tags to filter on (optional).
    """

    name: Optional[str]
    """
    Filter by secret name (optional).
    """

    path: Optional[str]
    """
    Filter by exact path (optional).
    """

    ephemeral: Optional[bool]
    """
    Filter by ephemeral / not ephemeral (optional).
    """

    type_: Optional[SecretType]
    """
    Filter by secret type (optional).
    """


@dataclass
class ListSecretsResponse:
    secrets: List[Secret]
    """
    Single page of secrets matching the requested criteria.
    """

    total_count: int
    """
    Count of all secrets matching the requested criteria.
    """


@dataclass
class ListTagsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListTagsResponse:
    tags: List[str]
    """
    List of tags.
    """

    total_count: int
    """
    Count of all tags matching the requested criteria.
    """


@dataclass
class ProtectSecretRequest:
    secret_id: str
    """
    ID of the secret to enable secret protection for.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class SSHKey:
    ssh_private_key: str
    """
    The private SSH key.
    """


@dataclass
class UnprotectSecretRequest:
    secret_id: str
    """
    ID of the secret to disable secret protection for.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class UpdateSecretRequest:
    secret_id: str
    """
    ID of the secret.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Secret's updated name (optional).
    """

    tags: Optional[List[str]]
    """
    Secret's updated list of tags (optional).
    """

    description: Optional[str]
    """
    Description of the secret.
    """

    path: Optional[str]
    """
    (Optional.) Location of the folder in the directory structure. If not specified, the path is `/`.
    """

    ephemeral_policy: Optional[EphemeralPolicy]
    """
    (Optional.) Policy that defines whether/when a secret's versions expire.
    """


@dataclass
class UpdateSecretVersionRequest:
    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be either:
- a number (the revision number)
- "latest" (the latest revision)
- "latest_enabled" (the latest enabled revision).
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    ephemeral_properties: Optional[EphemeralProperties]
    """
    (Optional.) Properties that defines the version's expiration date, whether it expires after being accessed once, and the action to perform (disable or delete) once the version expires.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from .types import (
    TokenScope,
    ContactPointEmail,
    ContactPoint,
    DataSource,
    GrafanaProductDashboard,
    GrafanaUser,
    Plan,
    Token,
    AlertManager,
    Grafana,
    ListContactPointsResponse,
    ListDataSourcesResponse,
    ListGrafanaProductDashboardsResponse,
    ListGrafanaUsersResponse,
    Alert,
    ListManagedAlertsResponse,
    ListPlansResponse,
    ListTokensResponse,
    Usage,
    UsageOverview,
    GlobalApiCreateGrafanaUserRequest,
    GlobalApiResetGrafanaUserPasswordRequest,
    GlobalApiSelectPlanRequest,
    GlobalApiSyncGrafanaDataSourcesRequest,
    RegionalApiCreateContactPointRequest,
    RegionalApiCreateDataSourceRequest,
    RegionalApiCreateTokenRequest,
    RegionalApiDeleteContactPointRequest,
    RegionalApiDisableAlertManagerRequest,
    RegionalApiDisableManagedAlertsRequest,
    RegionalApiEnableAlertManagerRequest,
    RegionalApiEnableManagedAlertsRequest,
    RegionalApiTriggerTestAlertRequest,
    RegionalApiUpdateDataSourceRequest,
)


def unmarshal_ContactPointEmail(data: Any) -> ContactPointEmail:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ContactPointEmail' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("to", None)
    if field is not None:
        args["to"] = field

    return ContactPointEmail(**args)


def unmarshal_ContactPoint(data: Any) -> ContactPoint:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ContactPoint' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("email", None)
    if field is not None:
        args["email"] = unmarshal_ContactPointEmail(field)
    else:
        args["email"] = None

    return ContactPoint(**args)


def unmarshal_DataSource(data: Any) -> DataSource:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DataSource' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("url", None)
    if field is not None:
        args["url"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("origin", None)
    if field is not None:
        args["origin"] = field

    field = data.get("synchronized_with_grafana", None)
    if field is not None:
        args["synchronized_with_grafana"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return DataSource(**args)


def unmarshal_GrafanaProductDashboard(data: Any) -> GrafanaProductDashboard:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'GrafanaProductDashboard' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("title", None)
    if field is not None:
        args["title"] = field

    field = data.get("url", None)
    if field is not None:
        args["url"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("variables", None)
    if field is not None:
        args["variables"] = field

    return GrafanaProductDashboard(**args)


def unmarshal_GrafanaUser(data: Any) -> GrafanaUser:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'GrafanaUser' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("login", None)
    if field is not None:
        args["login"] = field

    field = data.get("role", None)
    if field is not None:
        args["role"] = field

    field = data.get("password", None)
    if field is not None:
        args["password"] = field
    else:
        args["password"] = None

    return GrafanaUser(**args)


def unmarshal_Plan(data: Any) -> Plan:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Plan' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("sample_ingestion_price", None)
    if field is not None:
        args["sample_ingestion_price"] = field

    field = data.get("logs_ingestion_price", None)
    if field is not None:
        args["logs_ingestion_price"] = field

    field = data.get("traces_ingestion_price", None)
    if field is not None:
        args["traces_ingestion_price"] = field

    field = data.get("monthly_price", None)
    if field is not None:
        args["monthly_price"] = field

    field = data.get("retention_metrics_interval", None)
    if field is not None:
        args["retention_metrics_interval"] = field
    else:
        args["retention_metrics_interval"] = None

    field = data.get("retention_logs_interval", None)
    if field is not None:
        args["retention_logs_interval"] = field
    else:
        args["retention_logs_interval"] = None

    field = data.get("retention_traces_interval", None)
    if field is not None:
        args["retention_traces_interval"] = field
    else:
        args["retention_traces_interval"] = None

    return Plan(**args)


def unmarshal_Token(data: Any) -> Token:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Token' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("scopes", None)
    if field is not None:
        args["scopes"] = [TokenScope(v) for v in field] if field is not None else None

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("secret_key", None)
    if field is not None:
        args["secret_key"] = field
    else:
        args["secret_key"] = None

    return Token(**args)


def unmarshal_AlertManager(data: Any) -> AlertManager:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'AlertManager' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("alert_manager_enabled", None)
    if field is not None:
        args["alert_manager_enabled"] = field

    field = data.get("managed_alerts_enabled", None)
    if field is not None:
        args["managed_alerts_enabled"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("alert_manager_url", None)
    if field is not None:
        args["alert_manager_url"] = field
    else:
        args["alert_manager_url"] = None

    return AlertManager(**args)


def unmarshal_Grafana(data: Any) -> Grafana:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Grafana' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("grafana_url", None)
    if field is not None:
        args["grafana_url"] = field

    return Grafana(**args)


def unmarshal_ListContactPointsResponse(data: Any) -> ListContactPointsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListContactPointsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("contact_points", None)
    if field is not None:
        args["contact_points"] = (
            [unmarshal_ContactPoint(v) for v in field] if field is not None else None
        )

    field = data.get("has_additional_receivers", None)
    if field is not None:
        args["has_additional_receivers"] = field

    field = data.get("has_additional_contact_points", None)
    if field is not None:
        args["has_additional_contact_points"] = field

    return ListContactPointsResponse(**args)


def unmarshal_ListDataSourcesResponse(data: Any) -> ListDataSourcesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListDataSourcesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("data_sources", None)
    if field is not None:
        args["data_sources"] = (
            [unmarshal_DataSource(v) for v in field] if field is not None else None
        )

    return ListDataSourcesResponse(**args)


def unmarshal_ListGrafanaProductDashboardsResponse(
    data: Any,
) -> ListGrafanaProductDashboardsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListGrafanaProductDashboardsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("dashboards", None)
    if field is not None:
        args["dashboards"] = (
            [unmarshal_GrafanaProductDashboard(v) for v in field]
            if field is not None
            else None
        )

    return ListGrafanaProductDashboardsResponse(**args)


def unmarshal_ListGrafanaUsersResponse(data: Any) -> ListGrafanaUsersResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListGrafanaUsersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("grafana_users", None)
    if field is not None:
        args["grafana_users"] = (
            [unmarshal_GrafanaUser(v) for v in field] if field is not None else None
        )

    return ListGrafanaUsersResponse(**args)


def unmarshal_Alert(data: Any) -> Alert:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Alert' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("product_family", None)
    if field is not None:
        args["product_family"] = field

    field = data.get("product", None)
    if field is not None:
        args["product"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("rule", None)
    if field is not None:
        args["rule"] = field

    field = data.get("description", None)
    if field is not None:
        args["description"] = field

    return Alert(**args)


def unmarshal_ListManagedAlertsResponse(data: Any) -> ListManagedAlertsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListManagedAlertsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("alerts", None)
    if field is not None:
        args["alerts"] = (
            [unmarshal_Alert(v) for v in field] if field is not None else None
        )

    return ListManagedAlertsResponse(**args)


def unmarshal_ListPlansResponse(data: Any) -> ListPlansResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPlansResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("plans", None)
    if field is not None:
        args["plans"] = (
            [unmarshal_Plan(v) for v in field] if field is not None else None
        )

    return ListPlansResponse(**args)


def unmarshal_ListTokensResponse(data: Any) -> ListTokensResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("tokens", None)
    if field is not None:
        args["tokens"] = (
            [unmarshal_Token(v) for v in field] if field is not None else None
        )

    return ListTokensResponse(**args)


def unmarshal_Usage(data: Any) -> Usage:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Usage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("data_source_origin", None)
    if field is not None:
        args["data_source_origin"] = field

    field = data.get("data_source_type", None)
    if field is not None:
        args["data_source_type"] = field

    field = data.get("unit", None)
    if field is not None:
        args["unit"] = field

    field = data.get("quantity_over_interval", None)
    if field is not None:
        args["quantity_over_interval"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("data_source_id", None)
    if field is not None:
        args["data_source_id"] = field
    else:
        args["data_source_id"] = None

    field = data.get("interval", None)
    if field is not None:
        args["interval"] = field
    else:
        args["interval"] = None

    return Usage(**args)


def unmarshal_UsageOverview(data: Any) -> UsageOverview:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'UsageOverview' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("scaleway_metrics_usage", None)
    if field is not None:
        args["scaleway_metrics_usage"] = unmarshal_Usage(field)
    else:
        args["scaleway_metrics_usage"] = None

    field = data.get("scaleway_logs_usage", None)
    if field is not None:
        args["scaleway_logs_usage"] = unmarshal_Usage(field)
    else:
        args["scaleway_logs_usage"] = None

    field = data.get("external_metrics_usage", None)
    if field is not None:
        args["external_metrics_usage"] = unmarshal_Usage(field)
    else:
        args["external_metrics_usage"] = None

    field = data.get("external_logs_usage", None)
    if field is not None:
        args["external_logs_usage"] = unmarshal_Usage(field)
    else:
        args["external_logs_usage"] = None

    field = data.get("external_traces_usage", None)
    if field is not None:
        args["external_traces_usage"] = unmarshal_Usage(field)
    else:
        args["external_traces_usage"] = None

    return UsageOverview(**args)


def marshal_GlobalApiCreateGrafanaUserRequest(
    request: GlobalApiCreateGrafanaUserRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.login is not None:
        output["login"] = request.login

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.role is not None:
        output["role"] = str(request.role)

    return output


def marshal_GlobalApiResetGrafanaUserPasswordRequest(
    request: GlobalApiResetGrafanaUserPasswordRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_GlobalApiSelectPlanRequest(
    request: GlobalApiSelectPlanRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.plan_name is not None:
        output["plan_name"] = str(request.plan_name)

    return output


def marshal_GlobalApiSyncGrafanaDataSourcesRequest(
    request: GlobalApiSyncGrafanaDataSourcesRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_ContactPointEmail(
    request: ContactPointEmail,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.to is not None:
        output["to"] = request.to

    return output


def marshal_RegionalApiCreateContactPointRequest(
    request: RegionalApiCreateContactPointRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("email", request.email),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiCreateDataSourceRequest(
    request: RegionalApiCreateDataSourceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.type_ is not None:
        output["type"] = str(request.type_)

    return output


def marshal_RegionalApiCreateTokenRequest(
    request: RegionalApiCreateTokenRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.token_scopes is not None:
        output["token_scopes"] = [str(item) for item in request.token_scopes]

    return output


def marshal_RegionalApiDeleteContactPointRequest(
    request: RegionalApiDeleteContactPointRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("email", request.email),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiDisableAlertManagerRequest(
    request: RegionalApiDisableAlertManagerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiDisableManagedAlertsRequest(
    request: RegionalApiDisableManagedAlertsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiEnableAlertManagerRequest(
    request: RegionalApiEnableAlertManagerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiEnableManagedAlertsRequest(
    request: RegionalApiEnableManagedAlertsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiTriggerTestAlertRequest(
    request: RegionalApiTriggerTestAlertRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_RegionalApiUpdateDataSourceRequest(
    request: RegionalApiUpdateDataSourceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class DeploymentStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    CREATING = "creating"
    DEPLOYING = "deploying"
    READY = "ready"
    ERROR = "error"
    DELETING = "deleting"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ListDeploymentsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListModelsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    DISPLAY_RANK_ASC = "display_rank_asc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class NodeTypeStock(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STOCK = "unknown_stock"
    LOW_STOCK = "low_stock"
    OUT_OF_STOCK = "out_of_stock"
    AVAILABLE = "available"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class EndpointPrivateNetworkDetails:
    private_network_id: str
    """
    ID of the Private Network.
    """


@dataclass
class EndpointPublicAccessDetails:
    pass


@dataclass
class EndpointSpecPrivateNetwork:
    private_network_id: str
    """
    ID of the Private Network.
    """


@dataclass
class EndpointSpecPublic:
    pass


@dataclass
class Endpoint:
    id: str
    """
    Unique identifier.
    """

    url: str
    """
    For private endpoints, the URL will be accessible only from the Private Network.
In addition, private endpoints will expose a CA certificate that can be used to verify the server's identity.
This CA certificate can be retrieved using the `GetDeploymentCertificate` API call.
    """

    disable_auth: bool
    """
    Defines whether the authentication is disabled.
    """

    public_access: Optional[EndpointPublicAccessDetails]

    private_network: Optional[EndpointPrivateNetworkDetails]


@dataclass
class ModelS3Model:
    s3_url: str

    python_dependencies: Dict[str, str]

    node_type: Optional[str]

    triton_server_version: Optional[str]


@dataclass
class ACLRuleRequest:
    ip: str
    """
    It can be specified as a single IP address or a range of IP addresses in CIDR notation.
    """

    description: str
    """
    Description of the ACL rule.
    """


@dataclass
class ACLRule:
    id: str
    """
    Unique identifier.
    """

    ip: str
    """
    Allowed IP address or CIDR range.
    """

    description: str
    """
    Description of the ACL rule.
    """


@dataclass
class EndpointSpec:
    disable_auth: bool
    """
    By default, deployments are protected by IAM authentication.
When setting this field to true, the authentication will be disabled.
    """

    public: Optional[EndpointSpecPublic]

    private_network: Optional[EndpointSpecPrivateNetwork]


@dataclass
class Deployment:
    id: str
    """
    Unique identifier.
    """

    name: str
    """
    Name of the deployment.
    """

    project_id: str
    """
    Project ID.
    """

    status: DeploymentStatus
    """
    Status of the deployment.
    """

    tags: List[str]
    """
    List of tags applied to the deployment.
    """

    node_type: str
    """
    Node type of the deployment.
    """

    endpoints: List[Endpoint]
    """
    List of endpoints.
    """

    size: int
    """
    Current size of the pool.
    """

    min_size: int
    """
    Defines the minimum size of the pool.
    """

    max_size: int
    """
    Defines the maximum size of the pool.
    """

    model_name: str
    """
    The inference model used for the deployment.
    """

    model_id: str
    """
    ID of the model used for the deployment.
    """

    region: Region
    """
    Region of the deployment.
    """

    error_message: Optional[str]
    """
    Displays information if your deployment is in error state.
    """

    created_at: Optional[datetime]
    """
    Creation date of the deployment.
    """

    updated_at: Optional[datetime]
    """
    Last modification date of the deployment.
    """


@dataclass
class Model:
    id: str
    """
    Unique identifier.
    """

    name: str
    """
    Unique Name identifier.
    """

    project_id: str
    """
    Project ID.
    """

    provider: str
    """
    Name of the model provider.
    """

    tags: List[str]
    """
    List of tags applied to the model.
    """

    description: str
    """
    Purpose of the model.
    """

    has_eula: bool
    """
    Defines whether the model has an end user licence agreement.
    """

    region: Region
    """
    Region of the model.
    """

    is_public: bool
    """
    Defines whether the model is public or not.
    """

    compatible_node_types: List[str]
    """
    Names of the node types compatible with the model.
    """

    quantization_level: str
    """
    Quantization level of the model.
    """

    created_at: Optional[datetime]
    """
    Creation date of the model.
    """

    updated_at: Optional[datetime]
    """
    Last modification date of the model.
    """

    s3_model: Optional[ModelS3Model]


@dataclass
class NodeType:
    name: str
    """
    Name of the node type.
    """

    stock_status: NodeTypeStock
    """
    Current stock status for the node type.
    """

    description: str
    """
    Current specs of the offer.
    """

    vcpus: int
    """
    Number of virtual CPUs.
    """

    memory: int
    """
    Quantity of RAM.
    """

    vram: int
    """
    Quantity of GPU RAM.
    """

    disabled: bool
    """
    The node type is currently disabled.
    """

    beta: bool
    """
    The node type is currently in beta.
    """

    gpus: int
    """
    Number of GPUs.
    """

    region: Region
    """
    Region of the node type.
    """

    created_at: Optional[datetime]
    """
    Creation date of the node type.
    """

    updated_at: Optional[datetime]
    """
    Last modification date of the node type.
    """


@dataclass
class AddDeploymentACLRulesRequest:
    deployment_id: str
    """
    ID of the deployment to add ACL rules to.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    acls: Optional[List[ACLRuleRequest]]
    """
    List of ACL rules to add.
    """


@dataclass
class AddDeploymentACLRulesResponse:
    rules: List[ACLRule]
    """
    List of ACL rules added.
    """


@dataclass
class CreateDeploymentRequest:
    model_name: str
    """
    Name of the model to use.
    """

    node_type: str
    """
    Name of the node type to use.
    """

    endpoints: List[EndpointSpec]
    """
    List of endpoints to create.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name of the deployment.
    """

    project_id: Optional[str]
    """
    ID of the Project to create the deployment in.
    """

    accept_eula: Optional[bool]
    """
    If the model has an EULA, you must accept it before proceeding.
The terms of the EULA can be retrieved using the `GetModelEula` API call.
    """

    tags: Optional[List[str]]
    """
    List of tags to apply to the deployment.
    """

    min_size: Optional[int]
    """
    Defines the minimum size of the pool.
    """

    max_size: Optional[int]
    """
    Defines the maximum size of the pool.
    """


@dataclass
class CreateEndpointRequest:
    deployment_id: str
    """
    ID of the deployment to create the endpoint for.
    """

    endpoint: EndpointSpec
    """
    Specification of the endpoint.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteDeploymentACLRuleRequest:
    acl_id: str
    """
    ID of the ACL rule to delete.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteDeploymentRequest:
    deployment_id: str
    """
    ID of the deployment to delete.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class DeleteEndpointRequest:
    endpoint_id: str
    """
    ID of the endpoint to delete.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class Eula:
    content: str
    """
    Content of the end user licence agreement.
    """


@dataclass
class GetDeploymentCertificateRequest:
    deployment_id: str

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetDeploymentRequest:
    deployment_id: str
    """
    ID of the deployment to get.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetModelEulaRequest:
    model_id: str

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class GetModelRequest:
    model_id: str
    """
    ID of the model to get.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """


@dataclass
class ListDeploymentACLRulesRequest:
    deployment_id: str
    """
    ID of the deployment to list ACL rules for.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of ACL rules to return per page.
    """


@dataclass
class ListDeploymentACLRulesResponse:
    rules: List[ACLRule]
    """
    List of ACL rules on the current page.
    """

    total_count: int
    """
    Total number of ACL rules.
    """


@dataclass
class ListDeploymentsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of deployments to return per page.
    """

    order_by: Optional[ListDeploymentsRequestOrderBy]
    """
    Order in which to return results.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    name: Optional[str]
    """
    Filter by deployment name.
    """

    tags: Optional[List[str]]
    """
    Filter by tags.
    """


@dataclass
class ListDeploymentsResponse:
    deployments: List[Deployment]
    """
    List of deployments on the current page.
    """

    total_count: int
    """
    Total number of deployments.
    """


@dataclass
class ListModelsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    order_by: Optional[ListModelsRequestOrderBy]
    """
    Order in which to return results.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of models to return per page.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    name: Optional[str]
    """
    Filter by model name.
    """

    tags: Optional[List[str]]
    """
    Filter by tags.
    """


@dataclass
class ListModelsResponse:
    models: List[Model]
    """
    List of models on the current page.
    """

    total_count: int
    """
    Total number of models.
    """


@dataclass
class ListNodeTypesRequest:
    include_disabled_types: bool
    """
    Include disabled node types in the response.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    page: Optional[int]
    """
    Page number to return.
    """

    page_size: Optional[int]
    """
    Maximum number of node types to return per page.
    """


@dataclass
class ListNodeTypesResponse:
    node_types: List[NodeType]
    """
    List of node types.
    """

    total_count: int
    """
    Total number of node types.
    """


@dataclass
class SetDeploymentACLRulesRequest:
    deployment_id: str
    """
    ID of the deployment to set ACL rules for.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    acls: Optional[List[ACLRuleRequest]]
    """
    All existing ACL rules will be replaced by the new ones.
    """


@dataclass
class SetDeploymentACLRulesResponse:
    rules: List[ACLRule]
    """
    List of ACL rules that were set.
    """


@dataclass
class UpdateDeploymentRequest:
    deployment_id: str
    """
    ID of the deployment to update.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    name: Optional[str]
    """
    Name of the deployment.
    """

    tags: Optional[List[str]]
    """
    List of tags to apply to the deployment.
    """

    min_size: Optional[int]
    """
    Defines the new minimum size of the pool.
    """

    max_size: Optional[int]
    """
    Defines the new maximum size of the pool.
    """


@dataclass
class UpdateEndpointRequest:
    endpoint_id: str
    """
    ID of the endpoint to update.
    """

    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    disable_auth: Optional[bool]
    """
    By default, deployments are protected by IAM authentication.
When setting this field to true, the authentication will be disabled.
    """
